/***************************************************************************
                          kplayerprocess.h  -  description
                          ----------------
    begin                : Sat Jan 11 2003
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYERPROCESS_H
#define KPLAYERPROCESS_H

#include <qobject.h>

class KProcess;

/**Handles the MPlayer process invocation, input and output.
  *@author kiriuja
  */

class KPlayerProcess : public QObject
{
  Q_OBJECT

public: 
  /** The process states.
    */
  enum State
  {
    /** mplayer process is not running
      */
    Idle,
    /** mplayer process is running but has not started playing yet
      */
    Running,
    /** mplayer is playing the current file
      */
    Playing,
    /** mplayer is paused
      */
    Paused
  };

  /** The frame dropping options.
    */
  enum FrameDropping
  {
    /** no frame dropping
      */
    None,
    /** soft frame dropping
      */
    Soft,
    /** hard frame dropping
      */
    Hard
  };

  /** Constructs the objects and sets the initial property values.
    */
  KPlayerProcess (void);
  /** Frees up the allocated memory.
    */
  virtual ~KPlayerProcess();

  /** Returns the current file url or path.
    */
  QString url (void) const
    { return m_url; }
  /** Sets the current file url or path. Autoloads the subtitle file if it exists.
    */
  void setUrl (QString);
  /** Returns the current subtitle file url or path.
    */
  QString subtitleUrl (void) const
    { return m_subtitle; }
  /** Sets the current subtitle file url or path.
    */
  void setSubtitleUrl (const QString& url)
    { m_subtitle = url; }
  /** Returns the subtitle autoload option.
    */
  bool autoloadSubtitles  (void)
    { return m_autoload_subtitles; }
  /** Sets the subtitle autoload option.
    */
  void setAutoloadSubtitles  (bool autoload)
    { m_autoload_subtitles = autoload; }
  /** Returns the current window id.
    */
  int winId (void) const
    { return m_winId; }
  /** Sets the window id where video should be output.
    */
  void setWinId (int id)
    { m_winId = id; }

  /** Retrieves information on the current file.
    */
  bool get_info (void);
  /** Runs mplayer on the current file.
    */
  bool play (void);
  /** Pauses playback.
    */
  void pause (void);
  /** Stops playback.
    */
  void stop (void);
  /** Detaches the mplayer process.
    */
  void kill (void);
  /** The current process state.
    */
  KPlayerProcess::State state (void) const
    { return m_state; }
  /** Move to the time position given.
    */
  void absoluteSeek (int);
  /** Move by the time difference given.
    */
  void relativeSeek (int);
  /** Current volume level.
    */
  int volume (void) const
    { return m_volume; }
  /** Changes volume level.
    */
  void setVolume (int);
  /** Turns the sound off.
    */
  void mute (void);
  /** Current position in seconds.
    */
  float position (void) const
    { return m_position; }
  /** Time length in seconds.
    */
  float length (void) const
    { return m_length; }
  /** Video width.
    */
  int width (void) const
    { return m_width; }
  /** Video height.
    */
  int height (void) const
    { return m_height; }
  /** Video size.
    */
  QSize size (void) const
    { return QSize (m_width, m_height); }
  /** Frame dropping setting.
    */
  KPlayerProcess::FrameDropping frameDropping (void) const
    { return m_frame_drop; }
  /** Changes the frame dropping setting.
    */
  void setFrameDropping (KPlayerProcess::FrameDropping);
  /** Current brightness level.
    */
  int brightness (void) const
    { return m_brightness; }
  /** Changes brightness level.
    */
  void setBrightness (int);
  /** Current contrast level.
    */
  int contrast (void) const
    { return m_contrast; }
  /** Changes contrast level.
    */
  void setContrast (int);
  /** Current hue level.
    */
  int hue (void) const
    { return m_hue; }
  /** Changes hue level.
    */
  void setHue (int);
  /** Current saturation level.
    */
  int saturation (void) const
    { return m_saturation; }
  /** Changes saturation level.
    */
  void setSaturation (int);

protected:
  /** The pointer to the mplayer process object.
    */
  KProcess* m_player;
  /** The pointer to the mplayer process used to get file info.
    */
  KProcess* m_helper;

  // Following are options passed to mplayer
  /** The url or path of the current file.
    */
  QString m_url;
  /** The url or path of the current subtitle file.
    */
  QString m_subtitle;
  /** Window id for video output.
    */
  int m_winId;
  /** Audio output option.
    */
  char* m_ao;
  /** Video output option.
    */
  char* m_vo;
  /** Frame dropping.
    */
  KPlayerProcess::FrameDropping m_frame_drop;
  /** Sound enabled.
    */
  bool m_audio;
  /** Video enabled.
    */
  bool m_video;
  /** Video width.
    */
  int m_width;
  /** Video height.
    */
  int m_height;
  /** Sound volume.
    */
  int m_volume;
  /** Sound muted.
    */
  bool m_mute;
  /** Video brightness.
    */
  int m_brightness;
  /** Video contrast.
    */
  int m_contrast;
  /** Video hue.
    */
  int m_hue;
  /** Video saturation.
    */
  int m_saturation;
  /** Time length.
    */
  float m_length;
  /** Current position.
    */
  float m_position;
  /** Current state.
    */
  KPlayerProcess::State m_state;
  /** Subtitle autoloading.
    */
  bool m_autoload_subtitles;

  /** Plays the current file.
    */
  KProcess* run (void);
  /** Sets the process state and emits the stateChanged signal.
    */
  void setState (KPlayerProcess::State);

  // Following should really be private
  /** Current stdout line or part thereof
    */
  char* m_stdout_buffer;
  /** Current buffer length
    */
  int m_stdout_buffer_length;
  /** Current stdout line length
    */
  int m_stdout_line_length;
  /** Current helper stdout line or part thereof
    */
  char* m_helper_stdout_buffer;
  /** Current helper buffer length
    */
  int m_helper_buffer_length;
  /** Current helper stdout line length
    */
  int m_helper_line_length;
  int m_seek, m_seek_count;
  float m_helper_position;
  int m_ans_length;
  bool m_paused, m_quit;

  /** Converts a string to a floating point number, replacing a comma with a decimal point.
   */
  static float stringToFloat (QString);

protected slots:
  /** Receives notification when the mplayer process exits.
    */
  void onProcessExited (KProcess*);
  /** Receives notification when mplayer sends something to stdout.
    */
  void onReceivedStdout (KProcess*, char*, int);
  /** Receives notification when the mplayer helper process sends something to stdout.
    */
  void onReceivedHelperStdout (KProcess*, char*, int);
  /** Receives notification when mplayer sends something to stderr.
    */
  void onReceivedStderr (KProcess*, char*, int);

signals:
  /** Emitted when KPlayer state has changed.
    */
  void stateChanged (KPlayerProcess::State);
  /** Emitted when mplayer progress needs to be updated.
    */
  void progressChanged (float);
  /** Emitted when mplayer process used to get file information has finished.
    */
  void infoAvailable (void);
};

#endif
